package com.kupal.commons.cs;

import com.kupal.commons.cdm.Persistence;
import com.kupal.commons.cs.exception.ServiceException;

import java.util.List;

/**
 * @author Krauchenia Sergey
 *         Date: 21.07.2010
 *         Time: 15:13:11
 */
public interface Service<T extends Persistence> {

    /**
     * Persist the given instance.
     *
     * @param object the instance to save.
     * @throws ServiceException - in case of persistence errors
     */
    void save(final Persistence object) throws ServiceException;

    /**
     * search persistence object by specified example
     * @param example example of persistence object
     * @return list of persistence object
     * @throws ServiceException in case of persistence errors
     */
    List<T> search(T example) throws ServiceException;

    /**
     * Update the given persistent instance.
     *
     * @param object the instance to update
     * @return the instance that the state was merged to
     * @throws ServiceException - in case of persistence errors
     */
    T update(final T object) throws ServiceException;

    /**
     * Save the persistence instance if it's new or update the persistent instance if it exists.
     *
     * @param object the instance to save or update
     * @return the merged object if it is merged, or the persisted object for save action
     * @throws ServiceException - in case of persistence errors
     */
    T saveOrUpdate(final T object) throws ServiceException;

    /**
     * Merge the given persistent instance.
     *
     * @param object the instance to merge
     * @return the merged object
     * @throws ServiceException - in case of persistence errors
     */
    T merge(final T object) throws ServiceException;

    /**
     * Save the persistence instance if it's new or merge the persistent instance if it exists.
     *
     * @param object the instance to save or merge
     * @return the merged object if it is merged, or the persisted object for save action
     * @throws ServiceException - in case of persistence errors
     */
    T saveOrMerge(final T object) throws ServiceException;

    /**
     * Delete the given persistent instance.
     *
     * @param object The instance to delete
     * @throws ServiceException - in case of persistence errors
     */
    void delete(final Persistence object) throws ServiceException;

    /**
     * Load a persistent instance with the given id. Throw an unrecoverable exception if there is no matching database row.
     *
     * @param uidPk the persistent instance id.
     * @return the persistent instance
     * @throws ServiceException - in case of persistence errors
     */
    T load(final long uidPk) throws ServiceException;

    /**
     * Get a persistent instance with the given id. Return null if no matching record exists.
     *
     * @param uidPk            the persistent instance id.
     * @return the persistent instance
     * @throws ServiceException - in case of persistence errors
     */
    T get(final long uidPk) throws ServiceException;

    /**
     * Retrieve a list of persistent instances of the specified class with the specified property criteria.
     *
     * @param propertyCriteria property criteria for retrieving
     * @return list of persistence instance
     * @throws ServiceException - in case of persistence errors
     */
    List<T> find(final List propertyCriteria) throws ServiceException;

    /**
     * Update/delete all objects according to the given query.
     *
     * @param objects list to update
     * @return list of updated instances
     * @throws ServiceException - in case of persistence errors
     */
    List<T> bulkUpdate(final List<T> objects) throws ServiceException;

    /**
     * Lists all records stored in the database.
     *
     * @return a list of attribute
     * @throws ServiceException - in case of any errors
     */
    List<T> list() throws ServiceException;
}
